#!/bin/sh
#----------------------------------------------------------------------------
# /var/install/config.d/lm_sensors - change configuration of lm_sensors
#
# Creation:     2004-07-16 tomblom
# Last Update:  2024-03-16 Marcus Roeckrath
#
# Copyright (c) 2004 Thomas Unger
# Copyright (c) 2009-2013 Holger Bruenjes, holgerbruenjes(at)gmx(dot)net
# Copyright (c) 2015-2024 Marcus Roeckrath, marcus(dot)roeckrath(at)gmx(dot)net
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#----------------------------------------------------------------------------
# include eislib
. /var/install/include/eislib

# set packages name
package_name=lm_sensors
version=$(grep "version>" /var/install/packages/${package_name} |
          sed 's#</*version>##g')

# pathes and files
config_file=/etc/config.d/${package_name}
crontab_path=/var/cron/etc/root
crontab_file=${crontab_path}/${package_name}
mod_list=/etc/sysconfig/${package_name}
sensors_conf=/etc/sensors3.conf

# read config
. ${config_file}

#exec 2> /tmp/lm_sensors-trace$$.log
#set -x

# ---------------------------------------------------------------------------
# create sensors conf
# ---------------------------------------------------------------------------
create_sensors_conf ()
{
    # create new sensors3.conf
    rm -f /etc/sensors.conf
    myecho --info "Creating new ${sensors_conf}"
    echo "# ${sensors_conf}" > ${sensors_conf}

    if [ "${LM_SENSORS_CHIP_N}" -gt "0" ]
    then
    {
        # loop over chips
        idx=1
        while [ "${idx}" -le "${LM_SENSORS_CHIP_N}" ]
        do
            eval active='${LM_SENSORS_CHIP_'${idx}'_ACTIVE}'

            if [ $active = yes ]
            then
                eval name='${LM_SENSORS_CHIP_'${idx}'}'
                eval beep='${LM_SENSORS_CHIP_'${idx}'_BEEP}'
                echo 'chip "'${name}-*'"'
                if [ ${beep} = "yes" ]
                then
                    echo 'set beep_enable 1'
                else
                    if [ ${beep} = "no" ]
                    then
                      echo 'set beep_enable 0'
                    fi
                fi

                idx2=1
                eval count_feat_max='${LM_SENSORS_CHIP_'${idx}'_LINE_N}'
                while [ "${idx2}" -le "${count_feat_max}" ]
                do
                    eval value1='${LM_SENSORS_CHIP_'${idx}'_LINE_'${idx2}'}'
                    echo "${value1}"
                    idx2=$(expr ${idx2} + 1)
                done
            fi
            idx=$(expr ${idx} + 1)
        done
    } >> ${sensors_conf}
    fi
}

# ---------------------------------------------------------------------------
# create cron job
# ---------------------------------------------------------------------------
create_cron ()
{
    if [ "${LM_SENSORS_CRON_USE}" = "yes" ]
    then
        # check for cron directory
        if [ ! -d ${crontab_path} ]
        then
            mkdir -p ${crontab_path}
        fi

        if [ -f /usr/sbin/sendmail ]
        then
            myecho --info "Creating cronjob ..."

            if [ ! -f ${crontab_file} ]
            then
                # create cron job
                cat >${crontab_file} <<EOF
#--------------------------------------------------------------------
#  cron file generated by ${package_name} version: ${version}
#
#  Do not edit this file, edit ${config_file}
#  Creation Date: ${EISDATE} Time: ${EISTIME}
#--------------------------------------------------------------------
${LM_SENSORS_CRON_SCHEDULE} /var/install/config.d/lm_sensors.sh make_cron
EOF
                # update crontab file
                /var/install/config.d/cron >/dev/null 2>&1
            fi
        else
            myecho --warn "No mail package found."
            myecho --info "Please solve that first."
            set_parameter_no "LM_SENSORS_CRON_USE"
        fi
    else
        # delete existing file
        rm -f ${crontab_file}

        # update crontab file
        /var/install/config.d/cron >/dev/null 2>&1
    fi
}


# ---------------------------------------------------------------------------
# do cron
# ---------------------------------------------------------------------------
send_cron ()
{

/usr/bin/sensors > ${LM_SENSORS_OUTPUT}
alarm_mail=0
alarm_halt=0
while read line
do
    if echo ${line} | grep -q 'ALARM'
    then
        name=`echo ${line} | cut -d ":" -f 1 | sed -e 's/://g'`

        if [ ${LM_SENSORS_ALARM_MAIL_ON_N} -eq 0 ]
        then
            alarm_mail=1
        else
            imax=${LM_SENSORS_ALARM_MAIL_ON_N}

            idx=1
            while [ ${idx} -le ${imax} ]
            do
                eval value='${LM_SENSORS_ALARM_MAIL_ON_'${idx}'}'
                if [ "${value}" = "${name}" ]
                then
                    alarm_mail=1
                fi
                idx=`expr ${idx} + 1`
            done
        fi

        if [ ${LM_SENSORS_ALARM_HALT_ON_N} -eq 0 ]
        then
            alarm_halt=1
        else
            imax=${LM_SENSORS_ALARM_HALT_ON_N}
            idx=1
            while [ ${idx} -le ${imax} ]
            do
                eval chkname='${LM_SENSORS_ALARM_HALT_ON_'${idx}'}'
                if [ "${chkname}" = "${name}" ]
                then
                    eval maxval='${LM_SENSORS_ALARM_HALT_ON_'${idx}'_LIMIT}'
                    if [ -z "${maxval}" ]
                    then
                        alarm_halt=1
                    else
                        # wenn Temperatur
                        if echo ${line} | grep -qE '(.*):(.*).C[ ]*\((.*)=(.*).C,[ ]*(.*)=(.*).C\)(.*)'
                        then
                            value=`echo ${line} | sed 's/^\(.*\):[ ]*+\(.*\).C.[ ]*(\(.*\)=[ ]*\(.*\).C, \(.*\)=[ ]*\(.*\).C)[ ]*\(.*\)/\2/ ; s/^\(.*\)[.].*/\1/ ;'`
                            # wenn Wert hoeher als Limit
                            if [ ${value} -ge ${maxval} ]
                            then
                                alarm_halt=1
                            fi
                        # wenn Spannung
                        elif echo ${line} | grep -qE '(.*):(.*) V[ ]*\((.*)=(.*) V,[ ]*(.*)=(.*) V\)(.*)'
                        then
                            #value=`echo $zeile | sed 's/^\(.*\):[ ]*\(.*\) V[ ]*(\(.*\)=[ ]*\(.*\) V, \(.*\)=[ ]*\(.*\) V)[ ]*\(.*\)/\2/ ;'`
                           # Bei Spannungsalarm sofort anhalten
                            alarm_halt=1
                        # wenn Luefter
                        elif echo ${line} | grep -qE '(.*):(.*) RPM[ ]*\((.*)=(.*) RPM,(.*)=(.*)\)(.*)'
                        then
                            value=`echo ${line} | sed 's/^\(.*\):[ ]*\(.*\) RPM[ ]*(\(.*\)=[ ]*\(.*\) RPM, \(.*\)=[ ]*\(.*\))[ ]*\(.*\)/\2/ ;'`
                            # wenn Wert niedriger als Limit
                            if [ ${value} -le ${maxval} ]
                            then
                                alarm_halt=1
                            fi
                        else
                            mecho --error "ERROR READING SENSORS OUTPUT!!!"
                        fi
                    fi
                fi
                idx=`expr ${idx} + 1`
            done
        fi
    fi
done < ${LM_SENSORS_OUTPUT}

    if [ ${LM_SENSORS_ALARM_MAIL} = "yes" -a ${alarm_mail} -eq 1 ]
    then
        full_domain=$(hostname -f)
        # the message to send
    {
    echo "From: lm-sensors Alarm Agent <root@${full_domain}>"
    echo "To: <${LM_SENSORS_ALARM_MAIL_TO}>"
    echo "Subject: lm-sensors Alarm from Server '${HOSTNAME}'"
    echo "Mime-Version: 1.0"
    echo "X-Mailer: sendmail lm-sensors on eisfair"
    echo "Content-Type: text/plain; charset=iso-8859-1"
    echo "Content-Transfer-Encoding: quoted-printable"
    echo
    echo
    echo "Dispatched from lm_sensors on Server '${HOSTNAME}'"
    echo "Current Date: ${EISDATE} Time: ${EISTIME}"
    echo
    echo "Folgende Sensoren haben ihre zulaessigen Werte ueberschritten:"
    echo
    grep 'ALARM' < ${LM_SENSORS_OUTPUT} | sed -e 's/\ALARM//'
    echo
    if [ ${LM_SENSORS_ALARM_HALT} = "yes" -a ${alarm_halt} -eq 1 ]
    then
        echo "DAS SYSTEM WIRD ANGEHALTEN!!!"
    fi
    echo
    cat ${LM_SENSORS_OUTPUT}
    echo
    } | /usr/sbin/sendmail ${LM_SENSORS_ALARM_MAIL_TO}

    fi

    if [ ${LM_SENSORS_ALARM_HALT} = "yes" -a ${alarm_halt} -eq 1 ]
    then
        echo "Sensors alarm!"
        echo "System is going down!"
        halt
        exit 1
    fi

}

# ---------------------------------------------------------------------------
# delete cron job
# ---------------------------------------------------------------------------
delete_cron ()
{
    if [ -f ${crontab_file} ]
    then
        # delete existing file
        rm -f ${crontab_file}

        # update crontab file
        /var/install/config.d/cron >/dev/null 2>&1
    fi
}

# ---------------------------------------------------------------------------
# set parameter to no
# ---------------------------------------------------------------------------
set_parameter_no ()
{
    # ${1} rewrite Parameter from 'yes' to 'no'

    # set parameter from 'yes' to 'no'
    sed -i "s/^${1}=['\"].*['\"]/${1}='no' /" ${config_file}

}

# ---------------------------------------------------------------------------
# quiet suppress all normal output
# ---------------------------------------------------------------------------
myecho ()
{
    if ! ${_quiet:-false}
    then
        mecho ${@}
    fi
}

# ---------------------------------------------------------------------------
# main
# ---------------------------------------------------------------------------

while [ ${#} -ne 0 ]
do
    case ${1} in
    --quiet)
        _quiet=true
        shift
        ;;
    *)
        _action=${1}
        shift
        ;;
    esac
done

case "${_action}" in
    make_cron)
        send_cron
        ;;
    add_cron_job)
        create_cron
        ;;
    delete_cron_job)
        delete_cron
        ;;
    *)
        case ${START_LM_SENSORS} in
        yes)

            if [ ! -f /etc/sysconfig/lm_sensors ]
            then
                echo
                mecho --warn "/etc/sysconfig/lm_sensors not found!"
                mecho --info "  The Sensors have to be (re)detected."
                mecho --info "  /usr/sbin/sensors-detect will be started to detect your configuration."
                mecho --info "  Please follow the instructions and answer all questions of"
                mecho --info "  the detection script!"
                echo
                anykey
                clrhome
                /usr/sbin/sensors-detect
            fi

            create_sensors_conf
            /usr/sbin/service enable ${package_name}

            ;;
        *)
            /usr/sbin/service stop ${package_name}
            /usr/sbin/service disable ${package_name}
            ;;
        esac
        ;;
esac

exit 0
# ---------------------------------------------------------------------------
# end
# ---------------------------------------------------------------------------
